'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.runValidations = exports.getBodyData = exports.onError = exports.onRequest = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var onRequest = exports.onRequest = function () {
  var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(request, response) {
    var route, hasTimedOut, requestData, responseData, remoteAddress, handlerResult, result, statusCode;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            route = void 0;
            _context.prev = 1;
            hasTimedOut = false;

            response.setTimeout(_config2.default.timeout, function (socket) {
              var msg = 'Max request time of ' + _config2.default.timeout / 1000 + 's reached';
              (0, _response.sendResponse)(response, 408, new _errors2.default(408, msg));

              hasTimedOut = true;
              socket.destroy();
            });

            route = (0, _router.lookupRoute)({
              method: request.method,
              url: (0, _router.removeUrlQuery)(request.url)
            });

            if (route) {
              _context.next = 7;
              break;
            }

            throw new _errors2.default().notFound();

          case 7:
            route.validate = route.validate || {};
            route.statusCode = route.statusCode || 200;

            _context.next = 11;
            return getBodyData(request);

          case 11:
            _context.t0 = _context.sent;
            _context.t1 = route.validate.body;
            _context.t2 = stripUnknownData(_context.t0, _context.t1);
            _context.t3 = stripUnknownData((0, _router.getQueryData)(request), route.validate.query);
            _context.t4 = (0, _router.getParamData)(request.url, route.url);
            _context.t5 = request.headers;
            _context.t6 = {
              remoteAddress: (0, _utils.getIpAddress)(request)
            };
            requestData = {
              body: _context.t2,
              query: _context.t3,
              params: _context.t4,
              headers: _context.t5,
              info: _context.t6
            };
            responseData = {
              setHeader: function setHeader(name, value) {
                return response.setHeader(name, value);
              },
              redirect: function redirect(url) {
                return response._redirectTo = url;
              }
            };
            _context.next = 22;
            return runValidations(route, requestData);

          case 22:
            if (_config2.default.log.request) {
              remoteAddress = requestData.info.remoteAddress;

              console.log('=> ' + new Date().toISOString() + ' ' + request.method + ' ' + request.url + ' from ' + remoteAddress);
            }

            handlerResult = route.handler(requestData, responseData);

            if (!(handlerResult && handlerResult.then)) {
              _context.next = 30;
              break;
            }

            _context.next = 27;
            return handlerResult;

          case 27:
            _context.t8 = _context.sent;
            _context.next = 31;
            break;

          case 30:
            _context.t8 = handlerResult;

          case 31:
            _context.t7 = _context.t8;

            if (_context.t7) {
              _context.next = 34;
              break;
            }

            _context.t7 = { __noResponse: true };

          case 34:
            result = _context.t7;
            statusCode = (typeof result === 'undefined' ? 'undefined' : (0, _typeof3.default)(result)) === 'object' && result.statusCode ? result.statusCode : route.statusCode;

            if (!(statusCode >= 400 || result instanceof Error)) {
              _context.next = 38;
              break;
            }

            throw result;

          case 38:
            if (_config2.default.protect) {
              response.setHeader('X-Content-Type-Options', 'nosniff');
              response.setHeader('X-Download-Options', 'noopen');
              response.setHeader('X-Frame-Options', 'deny');
              response.setHeader('X-XSS-Protection', '1; mode=block');
            }

            if (!hasTimedOut) (0, _response.sendResponse)(response, statusCode, result);
            _context.next = 45;
            break;

          case 42:
            _context.prev = 42;
            _context.t9 = _context['catch'](1);

            onError(response, _context.t9, route);

          case 45:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this, [[1, 42]]);
  }));

  return function onRequest(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

var onError = exports.onError = function () {
  var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(response, error, route) {
    var onErrorResponse, _errorResponse, _statusCode, statusCode, errorResponse, toLog;

    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.prev = 0;

            if (!(route && typeof route.onError === 'function')) {
              _context2.next = 13;
              break;
            }

            onErrorResponse = route.onError(error);

            if (!onErrorResponse.then) {
              _context2.next = 9;
              break;
            }

            _context2.next = 6;
            return onErrorResponse;

          case 6:
            _context2.t0 = _context2.sent;
            _context2.next = 10;
            break;

          case 9:
            _context2.t0 = onErrorResponse;

          case 10:
            _errorResponse = _context2.t0;
            _statusCode = _errorResponse.statusCode || 500;
            return _context2.abrupt('return', (0, _response.sendResponse)(response, _statusCode, _errorResponse));

          case 13:
            statusCode = error.statusCode || 500;
            errorResponse = !error.error ? new _errors2.default(statusCode, error.message || error) : error;

            if (statusCode >= 500) {
              if (_config2.default.log.error) {
                toLog = error;

                if (!(toLog instanceof Error)) {
                  toLog = new Error(JSON.stringify(toLog));
                }

                console.error((0, _errors.cleanStackTrace)(toLog));
              }

              if (statusCode >= 500 && !_config2.default.isDev() && errorResponse.message) {
                delete errorResponse.message;
              }
            }

            (0, _response.sendResponse)(response, statusCode, errorResponse);
            _context2.next = 23;
            break;

          case 19:
            _context2.prev = 19;
            _context2.t1 = _context2['catch'](0);

            console.error(_context2.t1);
            (0, _response.sendResponse)(response, 500, new _errors2.default(500));

          case 23:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this, [[0, 19]]);
  }));

  return function onError(_x3, _x4, _x5) {
    return _ref2.apply(this, arguments);
  };
}();

var getBodyData = exports.getBodyData = function () {
  var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(request) {
    var contentType, mediaType, body;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            contentType = request.headers['content-type'] || 'text/plain; charset=utf-8';
            mediaType = _mediaTyper2.default.parse(contentType);
            body = {};

            if (!request.method.match(/^(post|put|patch)$/i)) {
              _context3.next = 7;
              break;
            }

            _context3.next = 6;
            return (0, _rawBody2.default)(request, {
              limit: _config2.default.payloadLimit,
              length: request.headers['content-length'],
              encoding: mediaType.parameters.charset || 'utf-8'
            });

          case 6:
            body = _context3.sent;

          case 7:
            _context3.t0 = mediaType.subtype;
            _context3.next = _context3.t0 === 'json' ? 10 : _context3.t0 === 'x-www-form-urlencoded' ? 18 : _context3.t0 === 'plain' ? 26 : 27;
            break;

          case 10:
            _context3.prev = 10;

            body = JSON.parse(body);
            _context3.next = 17;
            break;

          case 14:
            _context3.prev = 14;
            _context3.t1 = _context3['catch'](10);
            throw new _errors2.default().badRequest('Invalid JSON payload');

          case 17:
            return _context3.abrupt('break', 28);

          case 18:
            _context3.prev = 18;

            body = _querystring2.default.parse(body);
            _context3.next = 25;
            break;

          case 22:
            _context3.prev = 22;
            _context3.t2 = _context3['catch'](18);
            throw new _errors2.default().badRequest('Invalid form data');

          case 25:
            return _context3.abrupt('break', 28);

          case 26:
            return _context3.abrupt('break', 28);

          case 27:
            throw new _errors2.default().unsupportedMediaType();

          case 28:
            return _context3.abrupt('return', body);

          case 29:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this, [[10, 14], [18, 22]]);
  }));

  return function getBodyData(_x6) {
    return _ref3.apply(this, arguments);
  };
}();

var runValidations = exports.runValidations = function () {
  var _ref4 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4(route, data) {
    var allErrors, validation, validate, baseError, error;
    return _regenerator2.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            allErrors = [];
            validation = Object.assign({}, {
              body: {}, query: {}, params: {}
            }, route.validate);

            validate = function validate(key) {
              for (var i in validation[key]) {
                var validateResult = validation[key][i].run(i, data[key][i], key);
                if (validateResult.length > 0) allErrors = allErrors.concat(validateResult);
              }
            };

            validate('body');
            validate('query');
            validate('params');
            validate('headers');

            if (!(allErrors.length > 0)) {
              _context4.next = 11;
              break;
            }

            baseError = new _errors2.default().badRequest('A validation error occured');
            error = (0, _extends3.default)({}, baseError, { validation: allErrors });
            throw error;

          case 11:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, this);
  }));

  return function runValidations(_x7, _x8) {
    return _ref4.apply(this, arguments);
  };
}();

exports.stripUnknownData = stripUnknownData;

var _querystring = require('querystring');

var _querystring2 = _interopRequireDefault(_querystring);

var _mediaTyper = require('media-typer');

var _mediaTyper2 = _interopRequireDefault(_mediaTyper);

var _rawBody = require('raw-body');

var _rawBody2 = _interopRequireDefault(_rawBody);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _response = require('./response');

var _router = require('./router');

var _utils = require('./utils');

var _errors = require('./errors');

var _errors2 = _interopRequireDefault(_errors);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function stripUnknownData(data, validations) {
  if (!_config2.default.stripUnknown) return data;

  var newData = {};

  var keys = Object.keys(validations || {});
  if (keys.length === 0) return data;

  for (var i in data) {
    if (keys.indexOf(i) !== -1) {
      newData[i] = data[i];
    }
  }

  return newData;
}