'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

exports.sendResponse = sendResponse;

var _stream = require('stream');

var _stream2 = _interopRequireDefault(_stream);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function sendResponse(response, statusCode, data) {
  if (response._redirectTo) {
    response.statusCode = 301;
    response.setHeader('Location', response._redirectTo);
    response.end('Redirecting to ' + response._redirectTo);
    return;
  }

  response.statusCode = parseInt(statusCode, 10) || 200;
  response.setHeader('X-Powered-By', _config2.default.name);
  response.setHeader('Cache-Control', 'no-cache');

  if (data) {
    if (Buffer.isBuffer(data)) {
      response.setHeader('Content-Type', 'application/octet-stream');
      response.setHeader('Content-Length', data.length);
      response.end(data);
    } else if (data instanceof _stream2.default.Stream) {
      response.setHeader('Content-Type', 'application/octet-stream');
      data.pipe(response);
    } else {
      response.setHeader('Content-Type', 'text/plain');
      if ((typeof data === 'undefined' ? 'undefined' : (0, _typeof3.default)(data)) === 'object') {
        if (data.__noResponse) {
          data = '';
        } else {
          data = _config2.default.isDev() ? JSON.stringify(data, null, 2) : JSON.stringify(data);
          response.setHeader('Content-Type', 'application/json');
        }
      }

      response.setHeader('Content-Length', Buffer.byteLength(data));
      response.end(data);
    }
  } else {
    response.end();
  }
}