'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.expandUrl = expandUrl;
exports.getParamData = getParamData;
exports.getQueryData = getQueryData;
exports.removeUrlQuery = removeUrlQuery;
exports.lookupRoute = lookupRoute;
exports.createRoute = createRoute;
exports.getAllRoutes = getAllRoutes;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _querystring = require('querystring');

var _querystring2 = _interopRequireDefault(_querystring);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function expandUrl(url) {
  var esc = arguments.length <= 1 || arguments[1] === undefined ? true : arguments[1];

  if (typeof url !== 'string') {
    throw new TypeError('URL must be a string');
  }

  var split = url.split('/');

  return '^' + split.map(function (seg, idx) {
    var val = esc ? (0, _utils.escapeRegex)(seg) : seg;

    if (seg.match(/^:/)) val = '[^\\/\\?]+';

    if (idx === split.length - 1) val += '$';

    return val;
  }).join('\\/');
}

function getParamData(url, fromUrl) {
  if (typeof url !== 'string' || typeof fromUrl !== 'string') {
    throw new TypeError('URL must be a string');
  }

  var urlSplit = removeUrlQuery(url).split('/');
  var data = {};

  fromUrl.split('/').forEach(function (seg, indx) {
    var match = seg.match(/^:(.*)/);
    if (match) data[match[1]] = urlSplit[indx];
  });

  return data;
}

function getQueryData() {
  var request = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  if (typeof request.url !== 'string') {
    throw new TypeError('Param must be an object with a URL key');
  }

  var match = request.url.match(/\?.*$/);
  return match ? _querystring2.default.parse(match[0].substring(1)) : {};
}

function removeUrlQuery() {
  var url = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];

  var match = url.match(/\?/) || {};
  if (match.index) url = url.substring(0, match.index);

  return url;
}

function lookupRoute(route) {
  var methodMap = _config2.default.routeMap[route.method.toLowerCase()];

  var key = Object.keys(methodMap).find(function (r) {
    return route.url.match(new RegExp(r));
  });

  return methodMap[key];
}

function createRoute(route) {
  if (route.dir) {
    return getAllRoutes(route.dir, route.cwd);
  }

  if (Array.isArray(route)) {
    return route.forEach(createRoute);
  }

  if (!route) {
    throw new TypeError('Routes must have a configuration object');
  }
  if (!route.method.match(/^(get|post|put|patch|delete)$/i)) {
    throw new TypeError('Route method must be one of [ GET, POST, PUT, PATCH, DELETE ]');
  }
  if (typeof route.url !== 'string') {
    throw new TypeError('Route URL must be a string');
  }
  if (typeof route.handler !== 'function') {
    throw new TypeError('Route handler must be a function!');
  }

  var regexUrl = expandUrl(route.url);
  var method = route.method.toLowerCase();

  if (_config2.default.routeMap[method][regexUrl]) {
    throw new Error('"' + route.method + ' ' + route.url + '" is defined more than once');
  }

  _config2.default.routeMap[method][regexUrl] = route;
}

function getAllRoutes(dir, cwd) {
  dir = _path2.default.resolve(cwd || process.cwd(), dir);

  _glob2.default.sync(dir).forEach(function (file) {
    var routes = require(file);
    createRoute(routes.default || routes);
  });
}